/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002 - 2004
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: Composite.java,v 1.2 2004/08/12 10:01:06 dwagelaa Exp $
 * $Log: Composite.java,v $
 * Revision 1.2  2004/08/12 10:01:06  dwagelaa
 * * updated library support and better integration with CoCompose tool
 * * updated MainFrame interface
 *
 */

package EDU.utwente.CoCompose.lang;

import java.awt.*;
import java.io.File;
import java.util.*;

import javax.swing.*;

import EDU.auburn.VGJ.graph.Node;
import EDU.auburn.VGJ.util.*;
import EDU.utwente.CoCompose.CoCompose;
import EDU.utwente.CoCompose.util.*;
import EDU.utwente.CoCompose.translate.*;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.AttributesImpl;

/**
 * The CoCompose design language primitive Composite.
 *
 * @author Dennis Wagelaar
 * @version 0.1.22, 12 August 2004
 */

public class Composite extends ConstrainedConcept implements UniqueRealm
{
  //////////////////////////////////////////////////////////////////////////////
  // Constants
  //////////////////////////////////////////////////////////////////////////////

  private static final String XMLNAME_COMPOSITE       = "Composite";
  private static final String XMLNAME_ROLES           = "CompositeRoles";
  private static final String XMLNAME_PUBLISHED       = "PublishedConcepts";
  private static final String XMLNAME_SOLUTIONS       = "SolutionPatterns";
  private static final String XMLNAME_IMPLEMENTATIONS = "ImplementationGenerators";
  /**
   * Default Composite shape (rectangle).
   */
  protected static final int DefaultShape = RECTANGLE;
  /**
   * Default label position.
   */
  protected static final String DefaultLabelPos = "Below";

  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  private Identifier type_;
  private UniqueRealm typeRealm_;
  private ImageIcon icon_;
  private HashSet uniqueIds_ = new HashSet();
  private boolean showIconOnly_ = false;
  // The set of CompositePartConcepts contained by this Composite.
  private Vector concepts_ = new Vector();
  // The set of CompositeRoles contained by this Composite (Subset of concepts_).
  private Vector roles_ = new Vector();
  // The set of PublishedConcepts contained by this Composite (Subset of concepts_).
  private Vector published_ = new Vector();
  // The set of solution patterns for this Composite.
  private Vector solutions_ = new Vector();
  // The set of implementation generators for this Composite.
  private Vector implementations_ = new Vector();
  // Hashtable of language : String => implementations : Vector
  private Hashtable langImplementations_ = new Hashtable();

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a Composite.
   */
  public Composite()
  {
    super();
    // default to four times standard size
    width_ *= 4;
    height_ *= 4;
  } //Composite

  //////////////////////////////////////////////////////////////////////////////
  // Protected methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Draws the icon of the composite, if any.
   *
   * @param graphics The graphics context.
   * @param x The transformed X coordinate.
   * @param y The transformed Y coordinate.
   * @param scale The drawing scale.
   */
  protected void drawIcon(Graphics graphics, int x, int y, double scale)
  {
    // get icon, if any
    ImageIcon icon = getIcon();
    if (icon == null)
      return;

    double w = icon.getIconWidth() * scale;
    double h = icon.getIconHeight() * scale;

    graphics.drawImage(icon.getImage(), (int)(x - w / 2), (int)(y - h / 2),
      (int)w, (int)h, getFillColor(), null);
  } //drawIcon

  /**
   * Create XML from the composite roles using a SAX DefaultHandler.
   *
   * @param handler The SAX DefaultHandler.
   * @throws SAXException if an XML handler error occurs.
   */
  protected void rolesToXML(DefaultHandler handler) throws SAXException
  {
    Enumeration roles = getRoles();
    // if no roles, return
    if (!roles.hasMoreElements())
      return;

    handler.startElement(null, null, XMLNAME_ROLES, null);
    while (roles.hasMoreElements())
    {
      ((CompositeRole) roles.nextElement()).toXML(handler);
    }
    handler.endElement(null, null, XMLNAME_ROLES);
  } //rolesToXML

  /**
   * Create XML from the published concepts using a SAX DefaultHandler.
   *
   * @param handler The SAX DefaultHandler.
   * @throws SAXException if an XML handler error occurs.
   */
  protected void publishedToXML(DefaultHandler handler) throws SAXException
  {
    Enumeration concepts = getPublishedConcepts();
    // if no concepts, return
    if (!concepts.hasMoreElements())
      return;

    handler.startElement(null, null, XMLNAME_PUBLISHED, null);
    while (concepts.hasMoreElements())
    {
      ((PublishedConcept) concepts.nextElement()).toXML(handler);
    }
    handler.endElement(null, null, XMLNAME_PUBLISHED);
  } //publishedToXML

  /**
   * Create XML from the solution patterns using a SAX DefaultHandler.
   *
   * @param handler The SAX DefaultHandler.
   * @throws SAXException if an XML handler error occurs.
   */
  protected void solutionsToXML(DefaultHandler handler) throws SAXException
  {
    Enumeration solutions = getSolutions();
    // if no solutions, return
    if (!solutions.hasMoreElements())
      return;

    handler.startElement(null, null, XMLNAME_SOLUTIONS, null);
    while (solutions.hasMoreElements())
    {
      ((SolutionPattern) solutions.nextElement()).toXML(handler);
    }
    handler.endElement(null, null, XMLNAME_SOLUTIONS);
  } //solutionsToXML

  /**
   * Create XML from the implementation generators using a SAX DefaultHandler.
   *
   * @param handler The SAX DefaultHandler.
   * @throws SAXException if an XML handler error occurs.
   */
  protected void implementationsToXML(DefaultHandler handler) throws SAXException
  {
    Enumeration implementations = getImplementations();
    // if no implementations, return
    if (!implementations.hasMoreElements())
      return;

    handler.startElement(null, null, XMLNAME_IMPLEMENTATIONS, null);
    while (implementations.hasMoreElements())
    {
      ((ImplementationGenerator) implementations.nextElement()).toXML(handler);
    }
    handler.endElement(null, null, XMLNAME_IMPLEMENTATIONS);
  } //implementationsToXML

  /**
   * Create XML start from this object using a SAX DefaultHandler, element name
   * and given attributes.
   *
   * @param handler The SAX DefaultHandler.
   * @param element The XML element name.
   * @param atts The XML element attributes.
   * @throws SAXException if an XML handler error occurs.
   */
  protected void startXML(DefaultHandler handler, String element,
    AttributesImpl atts) throws SAXException
  {
    // create new attributes list if none given.
    if (atts == null)
      atts = new AttributesImpl();

    // Add attributes
    atts.addAttribute(null, null, "Type", "CDATA", getType());
    if (getShowIconOnly())
    {
      atts.addAttribute(null, null, "ShowIconOnly", "CDATA", "Yes");
    }
    else
    {
      atts.addAttribute(null, null, "ShowIconOnly", "CDATA", "No");
    }
    UniqueRealm realm = getUniqueRealm();
    Model model = getModel();
    if (realm != null && realm != model)
    {
      atts.addAttribute(null, null, "From", "CDATA", realm.getName());
    }

    // Write XML
    super.startXML(handler, element, atts);

    // CompositeRoles
    rolesToXML(handler);

    // PublishedConcepts
    publishedToXML(handler);

    // Solution patterns
    solutionsToXML(handler);

    // Implementation generators
    implementationsToXML(handler);
  } //startXML

  /**
   * Updates the language => implementations lookup table.
   */
  protected void updateLangImplTable()
  {
    Enumeration impls;
    ImplementationGenerator impl;
    Vector langImpls;

    langImplementations_.clear();
    impls = implementations_.elements();
    while (impls.hasMoreElements())
    {
      impl = (ImplementationGenerator) impls.nextElement();
      langImpls = (Vector) langImplementations_.get(impl.getLanguage());
      if (langImpls == null)
      {
        langImpls = new Vector();
        langImplementations_.put(impl.getLanguage(), langImpls);
      }
      langImpls.add(impl);
    }
  } //updateLangImplTable

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Sets the defaults for this composite.
   */
  public void setDefaults()
  {
    defaultLabel_ = "Composite";
    setShape(DefaultShape);
    setLabelPosition(DefaultLabelPos);
    // set default colors
    setEdgeColor(DEFAULT_EDGE_COLOR);
    setFillColor(DEFAULT_FILL_COLOR);
    setFontColor(DEFAULT_FONT_COLOR);
    setSelectColor(DEFAULT_SELECT_COLOR);
  } //setDefaults

  /**
   * Sets the label using the object attributes.
   */
  public synchronized void setLabel()
  {
    UniqueRealm realm = getUniqueRealm();
    Model model = getModel();
    String namespace = new String();
    // display namespace if not current model
    if (realm != null && realm != model)
    {
      namespace = "\\n(from " + realm.getName();
      if (namespace.length() > 16)
        namespace = namespace.substring(0, 16) + "...";
      namespace += ")";
    }
    // set label
    super.setLabel();
    if ((getType().length() > 0) && (!getShowIconOnly()))
    {
      setLabel(getLabel() + " : " + getType() + namespace);
    }
    else
    {
      setLabel(getLabel() + namespace);
    }
  } //setLabel

  /**
   * Sets the type uniqueness realm.
   *
   * @param realm The uniqueness realm for type identifiers.
   */
  public void setTypeUniqueRealm(UniqueRealm realm)
  {
    typeRealm_ = realm;
  } //setTypeUniqueRealm

  /**
   * Gets the type uniqueness realm.
   *
   * @return The uniqueness realm for type identifiers.
   */
  public UniqueRealm getTypeUniqueRealm()
  {
    return typeRealm_;
  } //getTypeUniqueRealm

  /**
   * Gets type attribute.
   *
   * @return The type attribute.
   */
  public String getType()
  {
    if (type_ != null)
      return type_.getValue();
    else
      return "";
  } //getType

  /**
   * Sets type attribute.
   *
   * @param type_in The new type.
   * @throws CoComposeException if the type is not a valid identifier.
   */
  public synchronized void setType(String type_in) throws CoComposeException
  {
    UniqueRealm realm = getTypeUniqueRealm();

    // set value
    if (type_ == null)
    {
      if (type_in.length() > 0)
      {
        type_ = new Identifier(null, type_in);
        // register value in type realm (type name may not be used for other ids)
        if (realm != null)
          getTypeUniqueRealm().register(type_in);
      }
    }
    else
    {
      // unregister value in type realm (type name may not be used for other ids)
      if (realm != null)
        realm.unRegister(type_.getValue());
      if (type_in.length() == 0)
        type_ = null;
      else
        type_.setValue(type_in);
      // register value in type realm (type name may not be used for other ids)
      if (realm != null)
        getTypeUniqueRealm().register(type_in);
    }
    // read icon as well, if it exists
    File iconFile = new File("icons/" + type_in + ".gif");
    if (iconFile.exists())
    {
      setIcon(new ImageIcon(iconFile.getAbsolutePath()));
    }
    // construct label by calling setName
    setLabel();
  } //setType

  /**
   * Sets the composite type icon.
   *
   * @param icon the composite type icon.
   */
  public void setIcon(ImageIcon icon)
  {
    icon_ = icon;
  } //setIcon

  /**
   * Gets the composite type icon.
   *
   * @return the composite type icon.
   */
  public ImageIcon getIcon()
  {
    return icon_;
  } //getIcon

  /**
   * Sets show icon only attribute.
   *
   * @param showIconOnly Whether or not to show only the composite icon.
   */
  public synchronized void setShowIconOnly(boolean showIconOnly)
  {
    showIconOnly_ = showIconOnly;
    setLabel();
    // update roles as well
    Enumeration roles = getRoles();
    while (roles.hasMoreElements())
    {
      ((CompositeRole) roles.nextElement()).setLabel();
    }
  } //setShowIconOnly

  /**
   * Gets show icon only attribute.
   *
   * @return Whether or not to show only the composite icon.
   */
  public boolean getShowIconOnly()
  {
    return showIconOnly_;
  } //getShowIconOnly

  /**
   * Adds a CompositePartConcept to this Composite and saves position offset.
   *
   * @param concept The CompositePartConcept to be added.
   */
  public void addPartConcept(CompositePartConcept concept)
  {
    concepts_.add(concept);
    if (concept instanceof CompositeRole)
      roles_.add(concept);
    else if (concept instanceof PublishedConcept)
      published_.add(concept);
    concept.setComposite(this);
    concept.saveCompositeOffset();
  } //addPartConcept

  /**
   * Removes a CompositePartConcept from this Composite.
   *
   * @param concept The CompositePartConcept to be removed.
   */
  public void removePartConcept(CompositePartConcept concept)
  {
    concepts_.remove(concept);
    if (concept instanceof CompositeRole)
      roles_.remove(concept);
    else if (concept instanceof PublishedConcept)
      published_.remove(concept);
  } //removePartConcept

  /**
   * Gets an enumeration of all the CompositePartConcepts in this composite.
   *
   * @return An Enumeration of the composite part concepts.
   */
  public Enumeration getPartConcepts()
  {
    return concepts_.elements();
  } //getPartConcepts

  /**
   * Checks whether this composite has the given CompositePartConcept.
   *
   * @param concept The composite part concept to check for.
   * @return True if and only if the composite contains the composite part
   *         concept.
   */
  public boolean hasPartConcept(CompositePartConcept concept)
  {
    return concepts_.contains(concept);
  } //hasPartConcept

  /**
   * Gets a CompositePartConcept in this composite by name.
   *
   * @param name The CompositePartConcept name.
   * @return The CompositePartConcept.
   */
  public CompositePartConcept getPartConcept(String name)
  {
    CompositePartConcept concept;
    CompositePartConcept result = null;

    Enumeration concepts = concepts_.elements();
    while (concepts.hasMoreElements())
    {
      concept = (CompositePartConcept) concepts.nextElement();
      if (concept.getName().equals(name))
      {
        result = concept;
        break;
      }
    }
    return result;
  } //getPartConcept

  /**
   * Gets a role in this composite by name.
   *
   * @param name The composite role name.
   * @return The role.
   */
  public CompositeRole getRole(String name)
  {
    CompositeRole role;
    CompositeRole result = null;

    Enumeration roles = roles_.elements();
    while (roles.hasMoreElements())
    {
      role = (CompositeRole) roles.nextElement();
      if (role.getName().equals(name))
      {
        result = role;
        break;
      }
    }
    return result;
  } //getRole

  /**
   * Gets an enumeration of all the roles in this composite.
   *
   * @return An Enumeration of the roles.
   */
  public Enumeration getRoles()
  {
    return roles_.elements();
  } //getRoles

  /**
   * Gets an enumeration of all the published concepts in this composite.
   *
   * @return An Enumeration of the published concepts.
   */
  public Enumeration getPublishedConcepts()
  {
    return published_.elements();
  } //getPublishedConcepts

  /**
   * Gets a published concept in this composite by name.
   *
   * @param name The published concept name.
   * @return The published concept.
   */
  public PublishedConcept getPublishedConcept(String name)
  {
    PublishedConcept concept;
    PublishedConcept result = null;

    Enumeration concepts = published_.elements();
    while (concepts.hasMoreElements())
    {
      concept = (PublishedConcept) concepts.nextElement();
      if (concept.getName().equals(name))
      {
        result = concept;
        break;
      }
    }
    return result;
  } //getPublishedConcept

  /**
   * Sets this composite as selected or unselected. Sets roles selection as well.
   *
   * @param selected When true, sets this composite as selected, otherwise
   *                 unselected.
   */
  public synchronized void setSelected(boolean selected)
  {
    super.setSelected(selected);
    // set part concepts to selected as well
    Enumeration concepts = getPartConcepts();
    while (concepts.hasMoreElements())
    {
      ((CompositePartConcept) concepts.nextElement()).setSelected(selected);
    }
  } //setSelected

  /**
   * Adds a solution pattern for this composite.
   *
   * @param solution The solution pattern to add.
   */
  public void addSolution(SolutionPattern solution)
  {
    solutions_.add(solution);
    solution.setComposite(this);
  } //addSolution

  /**
   * Inserts a solution pattern for this composite on a given index.
   *
   * @param solution The solution pattern to insert.
   * @param index The index on which to insert the solution pattern.
   */
  public void insertSolution(SolutionPattern solution, int index)
  {
    solutions_.insertElementAt(solution, index);
    solution.setComposite(this);
  } //insertSolution

  /**
   * Removes a solution pattern for this composite.
   *
   * @param solution The solution pattern to remove.
   */
  public void removeSolution(SolutionPattern solution)
  {
    solutions_.remove(solution);
    solution.setComposite(null);
  } //removeSolution

  /**
   * Gets the solution patterns for this composite.
   *
   * @return The solution patterns for this composite.
   */
  public Enumeration getSolutions()
  {
    return solutions_.elements();
  } //getSolutions()

  /**
   * Checks whether this composite has solution patterns.
   * @return True if it has solution patterns.
   */
  public boolean hasSolutions()
  {
    return !solutions_.isEmpty();
  } //hasSolutions

  /**
   * Adds an implementation generator for this composite.
   *
   * @param implementation The implementation generator to add.
   */
  public void addImplementation(ImplementationGenerator implementation)
  {
    implementations_.add(implementation);
    implementation.setComposite(this);
    updateLangImplTable();
  } //addImplementation

  /**
   * Inserts an implementation generator for this composite on a given index.
   *
   * @param implementation The implementation generator to insert.
   * @param index The index on which to insert the implementation generator.
   */
  public void insertImplementation(ImplementationGenerator implementation, int index)
  {
    implementations_.insertElementAt(implementation, index);
    implementation.setComposite(this);
    updateLangImplTable();
  } //insertImplementaton

  /**
   * Removes an implementation generator for this composite.
   *
   * @param implementation The implementation generator to remove.
   */
  public void removeImplementation(ImplementationGenerator implementation)
  {
    implementations_.remove(implementation);
    implementation.setComposite(null);
    updateLangImplTable();
  } //removeImplementation

  /**
   * Gets the implementation generators for this composite.
   *
   * @return The implementation generators for this composite.
   */
  public Enumeration getImplementations()
  {
    return implementations_.elements();
  } //getImplementations()

  /**
   * Gets the implementation generators for this composite given the language.
   *
   * @param language The language in which the implementation generators should be
   *                 expressed.
   * @return The implementation generators in the given language for this composite.
   */
  public Enumeration getImplementations(String language)
  {
    Vector langImpls = new Vector(), thisLangImpls;

    while (language != null)
    {
      thisLangImpls = (Vector) langImplementations_.get(language);
      if (thisLangImpls != null)
        langImpls.addAll(thisLangImpls);
      language = CoCompose.getSuperLanguage(language);
    }
    return langImpls.elements();
  } //getImplementations(String language)

  /**
   * Checks if there are implementation generators for this composite in the
   * given language.
   *
   * @param language The language in which the implementation generators should
   * 				 be expressed.
   * @return True if there are implementations generators.
   */
  public boolean hasImplementations(String language)
  {
    if (langImplementations_.get(language) != null)
    {
      return true;
    }
    else
    {
      String superLanguage = CoCompose.getSuperLanguage(language);
      if (superLanguage != null)
        return hasImplementations(superLanguage);
      else
        return false;
    }
  } //hasImplementations

  /**
   * Saves the offset coordinates from all roles and published concepts
   * relative to this composite.
   */
  public void saveOffsets()
  {
    Enumeration concepts = getPartConcepts();
    while (concepts.hasMoreElements())
    {
      ((CompositePartConcept) concepts.nextElement()).saveCompositeOffset();
    }
  } //saveOffsets

  /**
   * Restores the offset coordinates from all roles relative to this composite.
   */
  public void restoreOffsets()
  {
    // set label as well
    setLabel();
    Enumeration concepts = getPartConcepts();
    while (concepts.hasMoreElements())
    {
      ((CompositePartConcept) concepts.nextElement()).restoreCompositeOffset();
    }
  } //restoreOffsets

  /**
   * Copies attributes from fromComposite to self.
   *
   * @param fromComposite The Composite to copy the attributes from.
   * @throws CoComposeException if the copy could not be made.
   */
  public void copyAttributes(Composite fromComposite) throws CoComposeException
  {
    CompositeRole roleCopy, thisRole;
    PublishedConcept conceptCopy, thisConcept;
    SolutionPattern solutionCopy, thisSolution;
    ImplementationGenerator implementationCopy, thisImplementation;

    // set inherited attributes
    super.copyAttributes(fromComposite);
    // set own attributes
    try
    {
      setType(fromComposite.getType());
    }
    catch (CoComposeException e)
    { //should never happen
      e.printStackTrace();
    }
    setShowIconOnly(fromComposite.getShowIconOnly());
    // copy CompositeRoles
    Enumeration roles = fromComposite.getRoles();
    while (roles.hasMoreElements())
    {
      thisRole = (CompositeRole) roles.nextElement();
      roleCopy = new CompositeRole();
      roleCopy.copyAttributes(thisRole);
      addPartConcept(roleCopy);
    }
    // copy PublishedConcepts
    Enumeration concepts = fromComposite.getPublishedConcepts();
    while (concepts.hasMoreElements())
    {
      thisConcept = (PublishedConcept) concepts.nextElement();
      conceptCopy = new PublishedConcept();
      conceptCopy.copyAttributes(thisConcept);
      addPartConcept(conceptCopy);
      conceptCopy.setPublicName(thisConcept.getPublicName());
    }
    // copy SolutionPatterns
    Enumeration solutions = fromComposite.getSolutions();
    while (solutions.hasMoreElements())
    {
      thisSolution = (SolutionPattern) solutions.nextElement();
      solutionCopy = new SolutionPattern();
      solutionCopy.copyAttributes(thisSolution);
      addSolution(solutionCopy);
    }
    // copy ImplementationGenerators
    Enumeration implementations = fromComposite.getImplementations();
    while (implementations.hasMoreElements())
    {
      thisImplementation = (ImplementationGenerator) implementations.nextElement();
      implementationCopy = new ImplementationGenerator();
      implementationCopy.copyAttributes(thisImplementation);
      addImplementation(implementationCopy);
    }
  } //copyAttributes

  /**
   * Copies attributes from fromComposite to self. Copies type-sepcific attributes
   * only and resets the name and public names of published concepts.
   *
   * @param fromComposite The Composite to copy the attributes from.
   * @throws CoComposeException if the copy could not be made.
   */
  public void copyTypeAttributes(Composite fromComposite) throws CoComposeException
  {
    PublishedConcept thisConcept;

    // set inherited attributes
    copyAttributes(fromComposite);
    // reset name
    setName("");
    // reset PublishedConcepts public names
    Enumeration concepts = getPublishedConcepts();
    while (concepts.hasMoreElements())
    {
      thisConcept = (PublishedConcept) concepts.nextElement();
      thisConcept.setPublicName("");
    }
  } //copyTypeAttributes

  /**
   * Updates attributes from fromComposite to self. Assumes this composite is
   * already in use within a model and only needs the attributes copied, which
   * are specific for its type.
   *
   * @param fromComposite The Composite to copy the attributes from.
   * @throws CoComposeException if the copy could not be made.
   */
  public void updateAttributes(Composite fromComposite)
    throws CoComposeException
  {
    CompositeRole roleCopy, thisRole;
    PublishedConcept conceptCopy, thisConcept;
    SolutionPattern solutionCopy, thisSolution;
    ImplementationGenerator implementationCopy, thisImplementation;

    super.updateAttributes(fromComposite);
    setBoundingBox(fromComposite.getBoundingBox3());
    setSpecification(fromComposite.getSpecification());
    setSpecificationLang(fromComposite.getSpecificationLang());
    // set own attributes
    setType(fromComposite.getType());
    setShowIconOnly(fromComposite.getShowIconOnly());
    // copy CompositeRoles
    Enumeration roles = fromComposite.getRoles();
    while (roles.hasMoreElements())
    {
      thisRole = (CompositeRole) roles.nextElement();
      roleCopy = getRole(thisRole.getName());
      if (roleCopy == null)
      {
        roleCopy = new CompositeRole();
        roleCopy.copyAttributes(thisRole);
        addPartConcept(roleCopy);
      }
      else
      {
        roleCopy.saveCompositeOffset();
        roleCopy.copyAttributes(thisRole);
        roleCopy.restoreCompositeOffset();
      }
    }
    // copy PublishedConcepts
    Enumeration concepts = fromComposite.getPublishedConcepts();
    while (concepts.hasMoreElements())
    {
      thisConcept = (PublishedConcept) concepts.nextElement();
      conceptCopy = getPublishedConcept(thisConcept.getName());
      if (conceptCopy == null)
      {
        conceptCopy = new PublishedConcept();
        conceptCopy.copyAttributes(thisConcept);
        addPartConcept(conceptCopy);
      }
      else
      {
        conceptCopy.saveCompositeOffset();
        conceptCopy.copyAttributes(thisConcept);
        conceptCopy.restoreCompositeOffset();
      }
    }
    // copy SolutionPatterns
    solutions_.clear();
    Enumeration solutions = fromComposite.getSolutions();
    while (solutions.hasMoreElements())
    {
      thisSolution = (SolutionPattern) solutions.nextElement();
      solutionCopy = new SolutionPattern();
      solutionCopy.copyAttributes(thisSolution);
      addSolution(solutionCopy);
    }
    // copy ImplementationGenerators
    implementations_.clear();
    Enumeration implementations = fromComposite.getImplementations();
    while (implementations.hasMoreElements())
    {
      thisImplementation = (ImplementationGenerator) implementations.nextElement();
      implementationCopy = new ImplementationGenerator();
      implementationCopy.copyAttributes(thisImplementation);
      addImplementation(implementationCopy);
    }
  } //updateAttributes

  /**
   * Scales the size of the Composite. Also enforces boundary check on the
   * composite part concepts.
   *
   * @param scalex The X axis scale multiplier.
   * @param scaley The Y axis scale multiplier.
   * @param scalez The Z axis scale multiplier.
   */
  public void scale(double scalex, double scaley, double scalez)
  {
    Enumeration concepts;
    CompositePartConcept concept;

    // scale composite
    super.scale(scalex, scaley, scalez);
    // enforce part concept boundaries
    concepts = getPartConcepts();
    while (concepts.hasMoreElements())
    {
      concept = (CompositePartConcept) concepts.nextElement();
      concept.setPosition(concept.getPosition3());
    }
  } //scale

  /**
   * Draws the Composite on a component.
   *
   * @param comp The component to draw upon.
   * @param graphics The graphics context.
   * @param transform The coordinate transformation matrix.
   * @param quality The drawing quality.
   */
  public synchronized void draw(Component comp, Graphics graphics,
    Matrix44 transform, int quality)
  {
    // if in a group, don't draw - inherited.
    if ((isGroup_ && !groupActive_) || inActiveGroup_)
      return;

    // get scale
    double scale = transform.scale;

    // We scale the bounds here.
    Font font = new Font("Dialog", Font.PLAIN, (int) Math.round(FONT_SIZE * scale));
    graphics.setFont(font);
    scaleBounds(graphics, scale);

    double w = getBoundingBox().width * scale;
    double h = getBoundingBox().height * scale;

    if (w < 1) w = 1;
    if (h < 1) h = 1;

    DPoint3 position = getPosition3();
    position.transform(transform);

    int x = (int) position.x;
    int y = (int) position.y;

    // draw the shape
    drawShape(graphics, x, y, w, h);

    // draw the icon
    drawIcon(graphics, x, y, scale);

    // draw label if high quality requested
    if (quality > 0)
      drawLabel(graphics, x, y, h);

    // if not selected, we're done drawing
    if (!getSelected())
      return;

    // Draw selection components.
    drawSelected(graphics, position, scale);
  } //draw

  /**
   * Create XML from this object using a SAX DefaultHandler.
   *
   * @param handler The SAX DefaultHandler.
   * @throws SAXException if an XML handler error occurs.
   */
  public void toXML(DefaultHandler handler) throws SAXException
  {
    // Write XML
    startXML(handler, XMLNAME_COMPOSITE, null);
    endXML(handler, XMLNAME_COMPOSITE);
  } //toXML

  /**
   * Checks the completeness of the composite using default rules and the
   * specific CompletenessConstraint, if it exists.
   *
   * @throws CoComposeException if the composite is not completely specified.
   */
  public void checkCompleteness() throws CoComposeException
  {
    // check if type is null
    if (getType().length() == 0)
      throw new CoComposeException("The type of the " + getLabel() +
        " composite is not set", this);
    // inherited check
    super.checkCompleteness();
  } //checkCompleteness

  /**
   * Checks whether the given value is unique within the realm.
   *
   * @param value The value to check.
   * @return True if and only if the value is unique within the realm.
   */
  public boolean isUnique(String value)
  {
    return (!uniqueIds_.contains(value));
  } //isUnique

  /**
   * Registers a new value in the realm.
   *
   * @param value The value to register.
   */
  public void register(String value)
  {
    uniqueIds_.add(value);
  } //register

  /**
   * Unregisters a value from the realm.
   *
   * @param value The value to unregister.
   */
  public void unRegister(String value)
  {
    uniqueIds_.remove(value);
  } //unRegister

  /**
   * Gets the Postscript representation for the type image.
   *
   * @return The Postscript representation.
   */
  public String imagePS()
  {
    ImageIcon icon = getIcon();
    if (icon == null)
      return "";

    Image image = getIcon().getImage();
    int h = image.getHeight(null);
    int w = image.getWidth(null);
    int[] pixels = getImagePixels(image, w, h);
    StringBuffer result = new StringBuffer(100 + w * h * 12);
    int count;
    Color pixel;
    int pixelInt;
    int pixelR, pixelG, pixelB;
    String hexcode = "0";

    result.append("/image" + getId() + "  {\n\n");
    // make an image data block for each color component (r = 0, g = 1, b = 2)
    for (int colorComp = 0; colorComp < 3; colorComp++)
    {
      result.append("   < ");
      if (pixels != null)
      {
        count = 0;
        for (int row = 0; row < h; row++)
        {
          for (int col = 0; col < w; col++)
          {
            // wrap lines
            if ((count % 20) == 0 && count != 0)
              result.append("\n     ");
            pixelInt = pixels[count++];
            if ((pixelInt & 0xFF000000) == 0)
            { // transparent pixel, so set to background color
              pixel = getFillColor();
            }
            else
            {
              pixel = new Color(pixelInt);
            }
            switch (colorComp)
            {
              case 0: // red
                hexcode = Integer.toHexString(pixel.getRed());
                break;
              case 1: // green
                hexcode = Integer.toHexString(pixel.getGreen());
                break;
              case 2: // blue
                hexcode = Integer.toHexString(pixel.getBlue());
                break;
            }
            hexcode = hexcode.toUpperCase();
            // pad with a zero
            if (hexcode.length() < 2)
              hexcode = "0" + hexcode;
            result.append(hexcode + " ");
          } //for col
        } //for row
      }
      else
      {  // Error getting pixels.
        result.append("ff");  // All white.
      }
      result.append(">\n");
    } //for colorComp
    result.append("   ").append(w).append(" ").append(h).append("\n");
    result.append("}\ndef\n\n");
    return result.toString();
  } //imagePS

  /**
   * Create the PostScript invocation for the image.
   *
   * @param transform The coordinate transformation matrix.
   * @return The postscript representation.
   */
  public String toPSimage(Matrix44 transform)
  {
    ImageIcon icon = getIcon();
    if (icon == null)
      return "";

    DPoint3 pos = getPosition3();
    pos.transform(transform);
    double scale = transform.scale;
    Image image = icon.getImage();
    double w = image.getWidth(null) * scale;
    double h = image.getHeight(null) * scale;

    String result = PSnum_(pos.x) + PSnum_(pos.y) +
    PSnum_(w) + PSnum_(h) + "image" + getId() + " modelimage\n\n";
    return result;
  } //toPSimage

  /**
   * Accepts a ModelVisitor.
   *
   * @param visitor The visitor object to accept.
   * @throws CoComposeException if visiting operation failed.
   */
  public void accept(ModelVisitor visitor) throws CoComposeException
  {
    super.accept(visitor);
    visitor.visitComposite(this);
  } //accept

  /**
   * Checks whether this composite depends on other composites
   * through using their published concepts.
   * @param composite - the composite to check for dependencies.
   * @return true - if the composite has dependencies.
   */
  public boolean isDependent()
  {
    Model model = getModel();
    if (model == null)
      return false;
    Enumeration roles = getRoles();
    while (roles.hasMoreElements())
    {
      CompositeRole role = (CompositeRole) roles.nextElement();
      for (int child = role.firstChild(); child >= 0; child = role.nextChild())
      {
        Node link = model.getNodeFromIndex(child);
        if (link instanceof EDU.utwente.CoCompose.lang.PublishedConcept)
        {
          return true;
        }
      }
    }
    return false;
  } //isDependent
} //class Composite
