/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.lang;

import EDU.utwente.CoCompose.util.*;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.AttributesImpl;

/**
 * The abstract base class for concepts with completeness and correctness
 * constraints.
 *
 * @author Dennis Wagelaar
 * @version 0.1.18, 24 November 2003
 */

public abstract class ConstrainedConcept extends AbstractConcept
{
  //////////////////////////////////////////////////////////////////////////////
  // Constants
  //////////////////////////////////////////////////////////////////////////////

  // Path to constraint classes.
  private static final String CONSTRAINT_PATH = "constraints.";

  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  // constraint class names
  private String completenessConstraint_ = new String();
  private String correctnessConstraint_ = new String();

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a ConstrainedConcept
   */
  public ConstrainedConcept()
  {
    super();
  } //ConstrainedConcept

  //////////////////////////////////////////////////////////////////////////////
  // Protected methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Checks a constraint of this concept, if it exists.
   *
   * @param constraintName The class name of the constraint to check.
   * @throws CoComposeException if the constraint is not fulfilled.
   */
  protected void checkConstraint(String constraintName)
    throws CoComposeException
  {
    // check constraint, if set
    if (constraintName.length() > 0)
    {
      // constraint is set, try to load
      try
      {
        Constraint constraint = (Constraint)
          Class.forName(CONSTRAINT_PATH + constraintName).newInstance();
        constraint.check(this);
      }
      catch (Exception e)
      {
        throw new CoComposeException(e.getMessage(), this);
      }
    }
  } //checkConstraint

  /**
   * Create XML start from this object using a SAX DefaultHandler, element name
   * and given attributes.
   *
   * @param handler The SAX DefaultHandler.
   * @param element The XML element name.
   * @param atts The XML element attributes.
   * @throws SAXException if an XML handler error occurs.
   */
  protected void startXML(DefaultHandler handler, String element,
    AttributesImpl atts) throws SAXException
  {
    // create new attributes list if none given.
    if (atts == null)
      atts = new AttributesImpl();

    // Add attributes
    atts.addAttribute(null, null, "CompletenessConstraint", "CDATA",
      getCompletenessConstraint());
    atts.addAttribute(null, null, "CorrectnessConstraint", "CDATA",
      getCorrectnessConstraint());

    // Write XML
    super.startXML(handler, element, atts);
  } //startXML

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Gets the completeness constraint filename.
   *
   * @return The completeness constraint filename.
   */
  public String getCompletenessConstraint()
  {
    return completenessConstraint_;
  } //getCompletenessConstraint

  /**
   * Sets the completeness constraint filename.
   *
   * @param constraint_in The completeness constraint filename.
   */
  public void setCompletenessConstraint(String constraint_in)
  {
    completenessConstraint_ = constraint_in;
  } //setCompletenessConstraint

  /**
   * Gets the correctness constraint filename.
   *
   * @return The correctness constraint filename.
   */
  public String getCorrectnessConstraint()
  {
    return correctnessConstraint_;
  } //getCorrectnessConstraint

  /**
   * Sets the correctness constraint filename.
   *
   * @param constraint_in The correctness constraint filename.
   */
  public void setCorrectnessConstraint(String constraint_in)
  {
    correctnessConstraint_ = constraint_in;
  } //setCorrectnessConstraint

  /**
   * Checks the completeness of this concept using default rules and the
   * specific CompletenessConstraint, if it exists.
   *
   * @throws CoComposeException if this concept is not completely specified.
   */
  public void checkCompleteness() throws CoComposeException
  {
    // check if name is null
    if (getName().length() == 0)
      throw new CoComposeException("The name of " + getLabel() + " is not set",
        this);
    // check CompletenessConstraint
    checkConstraint(getCompletenessConstraint());
  } //checkCompleteness

  /**
   * Checks the correctness of this concept using the default rules and the
   * specific CorrectnessConstraint, if it exists.
   *
   * @throws CoComposeException if this concept is not correctly specified.
   */
  public void checkCorrectness() throws CoComposeException
  {
    // check CorrectnessConstraint
    checkConstraint(getCorrectnessConstraint());
  } //checkCorrectness

  /**
   * Copies attributes from fromConcept to self.
   *
   * @param fromConcept The Concept to copy the attributes from.
   */
  public void copyAttributes(ConstrainedConcept fromConcept)
  {
    super.copyAttributes(fromConcept);
    updateAttributes(fromConcept);
  } //copyAttributes

  /**
   * Updates attributes from fromConcept to self. Assumes this concept is
   * already in use within a model and only needs the attributes copied, which
   * are specific for this class.
   *
   * @param fromConcept The concept to copy the attributes from.
   */
  public void updateAttributes(ConstrainedConcept fromConcept)
  {
    setCorrectnessConstraint(fromConcept.getCorrectnessConstraint());
    setCompletenessConstraint(fromConcept.getCompletenessConstraint());
  } //updateAttributes

  /**
   * Accepts a ModelVisitor.
   *
   * @param visitor The visitor object to accept.
   * @throws CoComposeException if visiting operation failed.
   */
  public void accept(ModelVisitor visitor) throws CoComposeException
  {
    super.accept(visitor);
    visitor.visitConstrainedConcept(this);
  } //accept
} //class ConstrainedConcept
