/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002 - 2004
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.lang;

import java.awt.*;
import java.util.*;
import EDU.auburn.VGJ.graph.Node;
import EDU.utwente.CoCompose.util.CoComposeException;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * The CoCompose design language primitive SolutionRole. It is used to represent
 * composite roles in design language solution patterns.
 *
 * @author Dennis Wagelaar
 * @version 0.1.21, 6 April 2004
 */

public class SolutionRole extends AbstractConcept
{
  //////////////////////////////////////////////////////////////////////////////
  // Constants
  //////////////////////////////////////////////////////////////////////////////

  private static final String XMLNAME_ROLECONC = "SolutionRole";
  /**
   * Default label position.
   */
  protected static final String DefaultLabelPos = "Below";
  /**
   * The list of colors for identifying individual roles.
   */
  protected static final Color[] FILL_COLORS =
  {
    new Color(255, 204, 153), // orange
    new Color( 35, 225, 223), // cyan
    new Color(111, 114, 255), // purple
    new Color(111, 255, 111), // green
    new Color(154,  10,  10), // red
    new Color(250, 113,  30), // dark orange
    new Color(  0, 150, 255)  // blue
  };

  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  // Links to concepts that belong to this role part of a solution pattern.
  private HashSet concepts_ = new HashSet();
  private Color defaultFillColor_ = FILL_COLORS[0];
  private AbstractConcept filledBy_ = null;

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a SolutionRole.
   */
  public SolutionRole()
  {
    super();
  } //SolutionRole

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Sets the defaults for this solution role.
   */
  public void setDefaults()
  {
    defaultLabel_ = "SolutionRole";
    setShape(DefaultShape);
    setLabelPosition(DefaultLabelPos);
    // set default colors
    setEdgeColor(DEFAULT_EDGE_COLOR);
    setFillColor(DEFAULT_FILL_COLOR);
    setFontColor(DEFAULT_FONT_COLOR);
    setSelectColor(DEFAULT_SELECT_COLOR);
    // default to standard Concept size
    setBoundingBox(defaults.getBoundingBox3());
  } //setDefaults

  /**
   * Overwritten scale method to disallow scaling of this object. There's no
   * need to call this method, since it's a stub.
   *
   * @param scalex The X axis scale multiplier.
   * @param scaley The Y axis scale multiplier.
   * @param scalez The Z axis scale multiplier.
   */
  public void scale(double scalex, double scaley, double scalez)
  {
  } //scale

  /**
   * Gets an Iterator over all concepts belonging to this role part.
   */
  public Iterator getConcepts()
  {
    return concepts_.iterator();
  } //getConcepts

  /**
   * Adds a concept to this role part.
   *
   * @param concept The concept to add.
   */
  public void addConcept(AbstractConcept concept)
  {
    concepts_.add(concept);
  } //addConcept

  /**
   * Removes a concept from this role part.
   *
   * @param concept The concept to remove.
   */
  public void removeConcept(AbstractConcept concept)
  {
    concepts_.remove(concept);
  } //removeConcept

  /**
   * Sets the model this concept is part of and determines the fill color.
   *
   * @param model The model this concept is part of.
   */
  public synchronized void setModel(Model model)
  {
    // inherited setModel
    super.setModel(model);
    // if model is null, return
    if (model == null)
      return;
    // determine fill color by counting amount of role concepts
    int roleConceptCount = 0;
    for (Node thisNode = model.firstNode(); thisNode != null;
      thisNode = model.nextNode(thisNode))
    {
      if (thisNode instanceof EDU.utwente.CoCompose.lang.SolutionRole)
      {
        // Reset fill color to correct value, since role concepts may have
        // been deleted. This way there's more chance of keeping different
        // fill colors.
        ((SolutionRole) thisNode).setDefaultFillColor(
          FILL_COLORS[roleConceptCount % FILL_COLORS.length]);
        // increase counter
        roleConceptCount++;
      }
    }
    // choose next color
    setDefaultFillColor(FILL_COLORS[roleConceptCount % FILL_COLORS.length]);
  } //setModel

  /**
   * Overriden method stub, since a role concept can never be part of another
   * role concept.
   *
   * @return This role concept.
   */
  public SolutionRole getPartOf()
  {
    return this;
  } //getPartOf

  /**
   * Overriden method stub, since a role concept can never be part of another
   * role concept.
   *
   * @param role The solution role this concept is part of.
   */
  public synchronized void setPartOf(SolutionRole role)
  {
  } //setPartOf

  /**
   * Gets default fill color.
   *
   * @return The default fill color.
   */
  public Color getDefaultFillColor()
  {
    return defaultFillColor_;
  } //getDefaultFillColor

  /**
   * Sets the default fill color.
   *
   * @param color The default fill color.
   */
  public void setDefaultFillColor(Color color)
  {
    defaultFillColor_ = color;
    setFillColor(color);
  } //setDefaultFillColor

  /**
   * Gets the concept that fills the role described by this solution role. Note
   * that this is a volatile property that is only valid during flattening.
   *
   * @return filledBy_;
   */
  public AbstractConcept getFilledBy()
  {
    return filledBy_;
  } //getFilledBy

  /**
   * Sets the concept that fills the role described by this solution role. Note
   * that this is a volatile property that is only valid during flattening. It
   * should be unset (set to null) after applying a role part during flattening.
   *
   * @return filledBy_;
   */
  public void setFilledBy(AbstractConcept filled_by)
  {
    filledBy_ = filled_by;
  } //setFilledBy

  /**
   * Create XML from this object using a SAX DefaultHandler.
   *
   * @param handler The SAX DefaultHandler.
   * @throws SAXException if an XML handler error occurs.
   */
  public void toXML(DefaultHandler handler) throws SAXException
  {
    // Write XML
    startXML(handler, XMLNAME_ROLECONC, null);
    endXML(handler, XMLNAME_ROLECONC);
  } //toXML

  /**
   * Accepts a ModelVisitor.
   *
   * @param visitor The visitor object to accept.
   * @throws CoComposeException if visiting operation failed.
   */
  public void accept(ModelVisitor visitor) throws CoComposeException
  {
    super.accept(visitor);
    visitor.visitSolutionRole(this);
  } //accept
} //class SolutionRole
