/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.translate;

import java.util.*;
import EDU.utwente.CoCompose.lang.*;
import EDU.utwente.CoCompose.util.*;

/**
 * Checks the concept form constraints of the model. Supports incremental
 * checking, i.e. only checks for concepts with determined concept forms.
 * Concept checkming part.
 *
 * @author Dennis Wagelaar
 * @version 0.1.10, 5 June 2003
 */

public class ConceptFormConstraintsVisitor extends ModelVisitor
{
  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Parent implementation form constraint visitor.
   */
	private FormConstraintsVisitor parent;

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a ConceptFormConstraintsVisitor.
   * @param parent The parent visitor.
   */
  public ConceptFormConstraintsVisitor(FormConstraintsVisitor parent)
  {
    this.parent = parent;
  } //ConceptFormConstraintsVisitor

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Checks the concept form constraints for the concept.<BR>
   * PRE: The model must be defined for this concept.
   *
   * @param concept The concept to check the forms for.
   * @throws CoComposeException
   *   if one of the model's constraints are not fulfilled or
   *   if the concept form constraints are not met.
   */
  public void visitConcept(Concept concept) throws CoComposeException
  {
    Hashtable forms = parent.getForms();
    Model model = concept.getModel();
    String form;
    Enumeration impls;
    Implementation impl;
    boolean implWorks = true;

    if (log != null)
    {
      log.log("Checking concept form constraints for " + concept.getName());
      log.incLogLevel();
    }
    try
    {
      /* Precondition check -- disabled
      // check model
      if (model == null)
      {
        if (log != null)
          log.log("Error: " + concept.getName() + " has no model set");
        throw new Exception(concept.getName() + " has no model set");
      }
      End precondition check */
      // check if concept form determined, otherwise skip check
      if (forms.containsKey(concept))
      {
        form = (String) forms.get(concept);
        // check if we have implementations --
        if (concept.getImplementationCount() > 0)
          implWorks = false; // still to be proven
        impls = concept.getImplementations(model.getTargetLanguage());
        while (impls.hasMoreElements())
        {
          impl = (Implementation) impls.nextElement();
          if (form.equals(impl.getConceptForm()))
          { // implementation relevant - language and concept form apply
            // check implementation concept form constraints
            try
            {
              impl.accept(this);
              implWorks = true;
            }
            catch (CoComposeException e)
            { // try other implementations
            }
          }
        } //while
        if (!implWorks)
        {
          if (log != null)
            log.log("No implementation works for " + concept.getName());
          throw new CoComposeException("No implementation works for " +
            concept.getName(), concept);
        }
        // -- end implementation check
      } //if
    }
    finally
    {
      if (log != null)
        log.decLogLevel();
    }
  } //visitConcept

  /**
   * Check concept form constraints for this concept implementation.<BR>
   * PRE: The concept and model must be defined for the implementation.
   *
   * @param implementation The concept implementation to check.
   * @throws CoComposeException
   *   if one of the model's constraints are not fulfilled or
   *   if the concept form constraints are not met.
   */
  public void visitImplementation(Implementation implementation)
    throws CoComposeException
  {
    Hashtable forms = parent.getForms();
    Hashtable constraints = implementation.getConstraints();
    Collection constrForms;
    Iterator constrFs;
    String constrForm;
    boolean constrFulfilled;
    String form = implementation.getConceptForm();
    Concept concept = implementation.getConcept();
    Model model = implementation.getModel();
    String realm;
    AbstractConcept constrConcept;

    if (log != null)
    {
      log.log("Checking concept form constraints for concept implementation in " +
        implementation.getLanguage() + " as " + form);
      log.incLogLevel();
    }
    try
    {
      /* Precondition check -- disabled
      // check concept
      if (concept == null)
      {
        if (log != null)
          log.log("Error: implementation has no concept set");
        throw new Exception("Implementation has no concept set");
      }
      // check model
      if (model == null)
      {
        if (log != null)
          log.log("Error: implementation has no model set");
        throw new Exception("Implementation has no model set");
      }
      End precondition check */
      // get realm (every concept has one)
      realm = concept.getUniqueRealm().getName();
      // check constraints --
      Enumeration concepts = constraints.keys();
      while (concepts.hasMoreElements())
      {
        constrConcept = (AbstractConcept) concepts.nextElement();
        if (constrConcept instanceof EDU.utwente.CoCompose.lang.Concept)
        { // only check concepts with forms determined
          if (forms.containsKey(constrConcept))
          {
            String conceptForm = (String) forms.get(constrConcept);
            constrForms = (Collection) constraints.get(constrConcept);
            // check for each constraint concept form
//            if (!constrForms.contains(conceptForm))
            constrFs = constrForms.iterator();
            constrFulfilled = false;
            while ((constrFs.hasNext()) && (!constrFulfilled))
            {
              constrForm = (String) constrFs.next();
              // check whether this constrained concept has the required form or not
              if (constrForm.equals(conceptForm))
                constrFulfilled = true;
            } //while
            if (!constrFulfilled)
            { // not required form, so throw exception
              if (log != null)
                log.log(concept.getName() + " implementation constraints not met: form " + 
                  conceptForm + " not allowed for concept " + constrConcept.getName());
              throw new CoComposeException(concept.getName() +
                " implementation constraints not met: form " + 
                  conceptForm + " not allowed for concept " + constrConcept.getName(), 
                  concept);
            }
          }
        } //if constrConcept
      } //while
      // -- end constraints check
    }
    finally
    {
      if (log != null)
        log.decLogLevel();
    }
  } //visitImplementation
} //class ConceptFormConstraintsVisitor
