/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.xml;

import EDU.utwente.CoCompose.lang.Model;
import EDU.utwente.CoCompose.lang.CompositeType;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * XML reader for reading a CoCompose CompositeType XML document and create the
 * corresponding objects. Based on the SAX2 DefaultHandler class.
 *
 * @author Dennis Wagelaar
 * @version 0.0.10, 14 March 2002
 */

public class CompositeTypeReader extends ModelReader
{
  //////////////////////////////////////////////////////////////////////////////
  // Constants
  //////////////////////////////////////////////////////////////////////////////

  // XML Element names
  public static final String XMLNAME_TYPE = "CompositeType";
  // Mode value
  public static final int CREATING_TYPE = 6;

  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  /**
   * CompositeType to read into.
   */
  protected CompositeType compositeType;

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a CompositeTypeReader.
   *
   * @param compositeType_in The CompositeType object to build from the XML file.
   * @throws SAXException if the given CompositeType is null.
   */
  public CompositeTypeReader(CompositeType compositeType_in) throws SAXException
  {
    // inherited reader uses model to store nodes.
    super(new Model());
    if (compositeType_in == null)
    {
      throw new SAXException("Composite type cannot be null");
    }
    compositeType = compositeType_in;
  } //CompositeTypeReader

  //////////////////////////////////////////////////////////////////////////////
  // Protected methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Start CompositeType element.
   *
   * @param attrs Element attributes.
   * @throws Exception if the attributes could not be read.
   */
  protected void startCompositeType(Attributes attrs) throws Exception
  {
    // set newComposite_ for inherited composite role constructor
    newComposite_ = compositeType;
    startConstrainedConcept(newComposite_, attrs);
    String type = attrs.getValue("Type");
    if (type.length() > 0)
      newComposite_.setType(type);
    String iconOnly = attrs.getValue("ShowIconOnly");
    newComposite_.setShowIconOnly(iconOnly.equals("Yes"));
    newComposite_.setUniqueRealm(null); // set to bogus model, so reset
    mode_ = CREATING_TYPE;
  } //startCompositeType

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Start element.
   *
   * @param uri Ignored.
   * @param local Ignored.
   * @param raw Element name.
   * @param attrs Element attributes.
   * @throws SAXException if the new element could not be parsed.
   */
  public void startElement(String uri, String local, String raw,
    Attributes attrs) throws SAXException
  {
    try
    {
      // new CompositeType
      if (raw.equals(XMLNAME_TYPE))
        startCompositeType(attrs);
      // inherited element
      else
        super.startElement(uri, local, raw, attrs);
    }
    catch (Exception e)
    {
      throw new SAXException(e.getMessage());
    }
  } //startElement

  /**
   * End element.
   *
   * @param uri Ignored.
   * @param local Ignored.
   * @param raw Element name.
   * @throws SAXException if the new element could not be parsed.
   */
  public void endElement(String uri, String local, String raw)
    throws SAXException
  {
    // new CompositeType
    if (raw.equals(XMLNAME_TYPE))
    {
      mode_ = CREATING_NONE;
    }
    // new CompositeRole
    else if (raw.equals(XMLNAME_ROLE))
    {
      // embedded in CompositeType
      mode_ = CREATING_TYPE;
    }
    // PCDATA
    else if ((raw.equals(XMLNAME_GML))      ||
             (raw.equals(XMLNAME_COMMENTS)) ||
             (raw.equals(XMLNAME_SPEC)))
    {
      // check what we've read and process that
      switch (mode_)
      {
        case CREATING_TYPE:
          conceptChar(compositeType);
          break;
        default:
          // PCDATA for other element
          super.endElement(uri, local, raw);
          break;
      }
      pcdataMode_ = PCDATA_NONE;
    }
    // inherited element
    else
    {
      super.endElement(uri, local, raw);
    }
  } //endElement
} //class CompositeTypeReader
