/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.xml;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
import EDU.utwente.CoCompose.translate.*;

/**
 * XML reader for reading implementation generator data into an
 * ImplementationGenerator.
 *
 * @author Dennis Wagelaar
 * @version 0.1.02, 3 September 2002
 */

public class ImplementationGeneratorReader extends DefaultHandler
{
  //////////////////////////////////////////////////////////////////////////////
  // Constants
  //////////////////////////////////////////////////////////////////////////////

  // XML Element names
  public static final String XMLNAME_ROLEPART = "RolePart";
  public static final String XMLNAME_CONSTR   = "Constraint";

  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  /**
   * The implementation generator that is being read.
   */
  protected ImplementationGenerator pattern;
  /**
   * The role part that is currently being read.
   */
  protected ImplementationRolePart rolepart;

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs an ImplementationGeneratorReader.
   *
   * @param pattern_in The implementation generator object to read the data into.
   * @throws SAXException if the implementation generator is null.
   */
  public ImplementationGeneratorReader(ImplementationGenerator pattern_in)
    throws SAXException
  {
    if (pattern_in == null)
    {
      throw new SAXException("Implementation generator cannot be null");
    }
    pattern = pattern_in;
  } //ImplementationGeneratorReader

  //////////////////////////////////////////////////////////////////////////////
  // Protected methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Start RolePart element.
   *
   * @param attrs Element attributes.
   * @throws Exception if the attributes could not be read.
   */
  protected void startRolePart(Attributes attrs) throws Exception
  {
    rolepart = new ImplementationRolePart();
    rolepart.setRoleName(attrs.getValue("RoleName"));
    rolepart.setConceptForm(attrs.getValue("ConceptForm"));
  } //startRolePart

  /**
   * Start Constraint element.
   *
   * @param attrs Element attributes.
   * @throws Exception if the attributes could not be read.
   */
  protected void startConstraint(Attributes attrs) throws Exception
  {
    String rolename = attrs.getValue("RoleName");
    String notEqualTo = attrs.getValue("NotEqualTo");
    rolepart.addConstraint(rolename, notEqualTo);
  } //startConstraint

  /**
   * End RolePart element.
   */
  protected void endRolePart()
  {
    pattern.addRolePart(rolepart);
  } //endRolePart

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Start element.
   *
   * @param uri Ignored.
   * @param local Ignored.
   * @param raw Element name.
   * @param attrs Element attributes.
   * @throws SAXException if the new element could not be parsed.
   */
  public void startElement(String uri, String local, String raw,
    Attributes attrs) throws SAXException
  {
    try
    {
      // Role part
      if (raw.equals(XMLNAME_ROLEPART))
        startRolePart(attrs);
      // Constraint
      else if (raw.equals(XMLNAME_CONSTR))
        startConstraint(attrs);
    }
    catch (Exception e)
    {
      throw new SAXException(e.getMessage());
    }
  } //startElement

  /**
   * Characters.
   *
   * @param ch[] Character input.
   * @param start Starting index of characters.
   * @param length Length of characters.
   */
  public void characters(char ch[], int start, int length)
  {
    // for future use
  } //characters

  /**
   * End element.
   *
   * @param uri Ignored.
   * @param local Ignored.
   * @param raw Element name.
   * @throws SAXException if the new element could not be parsed.
   */
  public void endElement(String uri, String local, String raw)
    throws SAXException
  {
    // new RolePart
    if (raw.equals(XMLNAME_ROLEPART))
    {
      endRolePart();
    }
  } //endElement
} //class ImplementationGeneratorReader
